import React, { useState, useEffect, useRef, forwardRef, useImperativeHandle } from 'react';
import { Modal, Button, List, Popconfirm, Input } from 'antd';
import classNames from 'classnames';
import BasicForm from './BasicForm';
import { getPackageList, deletePackage } from '@/services/package';
import useBool from '@/utils/hooks/useBool';
import { floatDivide } from '@/utils/float';

export default forwardRef((props, ref) => {
  const { current, setCurrent } = props;
  const [loading, startLoading, stopLoading] = useBool();
  const [open, setVisible, hide] = useBool();
  const [hasMore, setHasMore] = useState(false);
  const [filter, setFilter] = useState({ pageNum: 1, pageSize: 20, total: 0 });
  const [list, setList] = useState([]);
  const listRef = useRef();
  listRef.current = list;

  const onLoadMore = () => {
    setFilter({
      ...filter,
      pageNum: filter.pageNum + 1,
    })
  }

  const loadMore = loading || !hasMore ? null : (
    <div
      style={{
        textAlign: 'center',
        marginTop: 12,
        height: 32,
        lineHeight: '32px',
      }}
    >
      <Button onClick={onLoadMore}>加载更多</Button>
    </div>
  );

  const onSearch = (val) => {
    setFilter({
      ...filter,
      name: val,
      pageNum: 1,
    })
  }

  const onEdit = (val = {}) => {
    setCurrent(val);
    setVisible();
  }

  const onDelete = (item) => {
    startLoading();
    deletePackage(item.id).then(res => {
      stopLoading();

      const newList = listRef.current.filter(x => x.id !== item.id);
      setList(newList);
      if (item.id === current.id) {
        setCurrent(newList[0] || {});
      }
    }).catch(() => {
      stopLoading();
    });
  };

  const handleForm = (val) => {
    setCurrent(val);
    hide()
  }

  useEffect(() => {
    startLoading();
    getPackageList(filter).then(res => {
      stopLoading();

      const { records = [], current, total, pages } = res

      if (current === 1) {
        setList(records)
        setCurrent((records || [])[0] || {})
      } else {
        setList(listRef.current.concat(records))
      }

      setHasMore(current < pages)
    }).catch(() => {
      stopLoading();
    });
  }, [filter]);

  useEffect(() => {
    if (!current || !current.id) return;

    let found = false;
    let newList = listRef.current.map(x => {
      if (x.id === current.id) {
        found = true;
        return current;
      }

      return x;
    })

    if (!found) {
      newList = newList.concat(current);
    }

    setList(newList);
  }, [current])

  useImperativeHandle(ref, () => ({
    new: () => onEdit(),
  }));

  return (
    <>
    <List
      style={{ minHeight: '600px' }}
      bordered
      header={<Input.Search allowClear placeholder="请输入筛选名称" onSearch={onSearch} />}
      dataSource={list}
      loading={loading}
      loadMore={loadMore}
      renderItem={item => {
        const price = item.price ? floatDivide(item.price, 100) : 0;

        return (
          <List.Item
            className={classNames({ active: current.id === item.id })}
            onClick={() => setCurrent(item)}
            actions={[
              <Button key="edit" type='link' onClick={() => onEdit(item)}>编辑</Button>,
              <Popconfirm
                key="delete"
                title="确认删除?"
                onConfirm={() => onDelete(item)}
                >
                <a href="#">删除</a>
              </Popconfirm>
            ]}
          >
            {`${item.name} ￥${price}元`}
          </List.Item>
        )
      }}
    />
    <Modal open={open} title="套餐维护" footer={null} onCancel={hide} maskClosable={false} >
      <BasicForm current={current} onChange={handleForm}></BasicForm>
    </Modal>
    </>
  )
})
